


// James McCartney


s.boot;

(
// define a simple reverb.
SynthDef("reverb", {
	var in;
	in = In.ar(0, 2);
	5.do({ in = AllpassN.ar(in, 0.05, [0.05.rand, 0.05.rand], 2); });
	ReplaceOut.ar(0, in);
}).load(s);
)


(
// define 100 randomly generated percussive FM instruments.
// this will take a few seconds.
// this code can be modified to generate sustaining instruments.
var carrierF, middleF, modulatorF;

carrierF = { arg freq=440, mod=0, mix=0, gate=1, envDur=2.0, minAttack = 0.001, maxAttack = 0.4;
	var e, m;
	// e = Env.adsr(exprand(0.001,maxAttack), linrand(3.0), rrand(0.4,1.0).squared, rrand(0.001,0.2));
	e = Env.perc(Rand(minAttack,maxAttack), Rand(2.0,envDur));
	// e = Env.perc(1, 1.01, 1, 4);
	m = linrand(10) + 1;
	e = EnvGen.kr(e, gate,  rrand(0.5,0.6).rand.squared);
	SinOsc.ar(freq * m, mod, e, mix);
};

middleF = { arg freq=440, mod=0, mix=0, gate=1, envDur=2.0, minAttack = 0.001, maxAttack = 0.4;
	var e, m;
	//e = Env.adsr(exprand(0.001,maxAttack), linrand(3.0), 1.0.rand.squared, rrand(0.001,0.2));
	e = Env.perc(Rand(minAttack,maxAttack), Rand(0.1,envDur));
	// e = Env.perc(1, 1.01, 1, 4);
	m = linrand(5) + 1;
	e = EnvGen.kr(e, gate,  3.0.rand.squared);
	SinOsc.ar(freq * m, mod, e, mix);
};

modulatorF = { arg freq=440, mix=0, gate=1, envDur=2.0, minAttack = 0.001, maxAttack = 0.4;
	var e, m;
	//e = Env.adsr(exprand(0.001,maxAttack), linrand(3.0), 1.0.rand.squared, rrand(0.001,0.2));
	e = Env.perc(Rand(minAttack,maxAttack), Rand(0.1,envDur));
	// e = Env.perc(1, 1.01, 1, 4);
	m = linrand(5) + 1;
	e = EnvGen.kr(e, gate,  3.0.rand.squared);
	SinOsc.ar(freq * m, 1.3.rand.cubed, e, mix);
};

100.do { |i|
	var name;
	name = "fmgen_a_" ++ i;
	SynthDef(name, { arg freq=440, amp=1, gate=1, pan=0;
		var c, m;

		c = 0;
		[
			{
				// sum of 3 modulator->carrier pairs
				[\A, i].postln;
				3.do {
					var f;
					f = freq + 1.8.rand2.squared;
					m = modulatorF.(f, 0, gate);
					c = carrierF.(f, m, c, gate);
				}
			},
			{
				// sum of 2 modulator->modulator->carrier chains
				[\B, i].postln;
				2.do {
					var f, m;
					f = freq + 1.8.rand2.squared;
					m = modulatorF.(f, 0, gate);
					m = middleF.(f, m, 0, gate);
					c = carrierF.(f, m, c, gate);
				}
			},
			{
				// sum of 2 modulator-+->carrier
				//                    |
				//                    +->carrier
				[\C, i].postln;
				2.do {
					var f;
					f = freq + 1.8.rand2.squared;
					m = modulatorF.(f, 0, gate);
					c = carrierF.(f, m, c, gate);
					c = carrierF.(f, m, c, gate);
				}
			},
		].choose.value;

		DetectSilence.ar(c, doneAction: 2);
		Out.ar(0, Pan2.ar(c, pan, amp));
	}).load(s);
};



)


(
// listen randomly to the random FM instruments.
r = Task({
	var name, freqMIDI = 60, freq, mySpeed = "fast", myDo = 100, myEnv = 2.0, myMaxAttack, myMinAttack;
	s.sendMsg(\s_new, \reverb, 1000, 1, 0);
	name = "fmgen_a_" ++ 100.rand;
	name.postln;

	loop {
		if(mySpeed == "fast", { myDo = 20 }, { myDo = 20 });
			// do
			(myDo.rand).postln.do { arg i;
				var id, freqChange, freqChangeFraction;
				// change the sound, maybe...
				if (0.5.coin) { name = "fmgen_a_" ++ 100.rand; };

				// chord unit
				[1,1,1,1,1,1,2,2,3,6,12,23,].choose.postln.do {

					// pitch unit
					freqChangeFraction = [ 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, -0.1, -0.2, -0.3, -0.4, -0.5, -0.6, -0.7, -0.8, -0.9 ].choose;
					freqChange = [ -11, -3, -2, -1, 1, 2, 3, 11 ].choose;
					freqMIDI = (freqMIDI + freqChange) + freqChangeFraction;
					freqMIDI = freqMIDI % 127;
					freqMIDI.postln;
					freq = freqMIDI.midicps;
					// end of pitch unit

					// env unit
					id = 1000000000.rand;
					if(mySpeed == "fast", { myEnv = 2.0; myMaxAttack = 1.4; myMinAttack = 0.001; }, { myEnv = 32.0; myMaxAttack = 10.8; myMinAttack = 2.0; });
					s.sendMsg(\s_new, name, id, 0, 0, \freq, freq, \amp, 0.4, \pan, 1.0.rand2, \envDur, myEnv, \maxAttack, myMaxAttack, \minAttack, myMinAttack);
					// end of env unit
				}; // end of chord unit

				// duration unit
				if(mySpeed == "fast", { [0.05, 0.04, 0.03, 0.02, 0.01, 0.1, 0.2, 0.3].choose.wait; }, { [0.01, 0.4, 0.5, 0.8, 1.2, 1.3, 3.0, 4.0, 6.0, 7.0].choose.wait; });
				// end of duration unit
			};
			// end of do
			if(0.5.coin, { mySpeed = "fast" } , { mySpeed = "slow" });
			"----------------------------------------------------".postln;
			mySpeed.postln;
			myDo.postln;
			"----------------------------------------------------".postln;
	}; // end of loop
	s.sendMsg(\n_free, 1000);
}); // end of task
)

r.play;
r.stop;

